/*start==============================================================================================================================================================================================
   Automatische Steuerung zum Maischen

   Überarbeitung "Bitter" (Lothar) und Borsti84 (Sebastian)

   V3.05R - Relaisversion für Display LCD 20x4 I2C und Gradientenregelung 

   Version vom 20.6.16 - Clickencoder Bibliothek gewechselt



   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
   Die Software wird unter der MIT - Lizenz veröffentlicht

   Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files ,
   to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
   WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

   Hiermit wird unentgeltlich jeder Person, die eine Kopie der Software und der zugehörigen Dokumentationen erhält, die Erlaubnis erteilt,
   sie uneingeschränkt zu benutzen, inklusive und ohne Ausnahme dem Recht, sie zu verwenden, kopieren, ändern, fusionieren, verlegen, verbreiten, unterlizenzieren
   und/oder zu verkaufen, und Personen, die diese Software erhalten, diese Rechte zu geben, unter den folgenden Bedingungen:
   Der obige Urheberrechtsvermerk und dieser Erlaubnisvermerk sind in allen Kopien oder Teilkopien der Software beizulegen.
   DIE SOFTWARE WIRD OHNE JEDE AUSDRÜCKLICHE ODER IMPLIZIERTE GARANTIE BEREITGESTELLT, EINSCHLIESSLICH DER GARANTIE ZUR BENUTZUNG FÜR DEN VORGESEHENEN ODER
   EINEM BESTIMMTEN ZWECK SOWIE JEGLICHER RECHTSVERLETZUNG, JEDOCH NICHT DARAUF BESCHRÄNKT. IN KEINEM FALL SIND DIE AUTOREN ODER COPYRIGHTINHABER FÜR JEGLICHEN
   SCHADEN ODER SONSTIGE ANSPRÜCHE HAFTBAR ZU MACHEN, OB INFOLGE DER ERFÜLLUNG EINES VERTRAGES, EINES DELIKTES ODER ANDERS IM ZUSAMMENHANG MIT DER
   SOFTWARE ODER SONSTIGER VERWENDUNG DER SOFTWARE ENTSTANDEN.
   xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

  Über diese Version

    Alle Einstellungen sind mit dem Encoder vorzunehmen.
      - Encoder drehen = Werteverstellung
      - Encoder schnell drehen = schnelle Werteverstellung
      - kurz Drücken = Eingabebestätigung
      - Doppelclick innerhalb von 0,6 s = Setupmenue -> Zurück -> lfd. Programm
        So kann z.B. während des Maischens im Setup-Menü der Gradientenfaktor geändert werden. Während der Zeit im Setup-Menü stoppt der Zeitablauf!
      - lang Drücken (> 4 Sekunden) = Abbruch und zurück zum Startmenü
      - bei Abbruch werden alle eingegebenen Werte beibehalten
    Wenn man z.B. einen Fehler bei der Eingabe macht, einfach lange drücken bis zum Abbruch. Dann die Eingaben durch drücken und drehen ggf. verändern.
      - während des automatischen Ablaufs sind durch Drehen des Encoders Änderungen an der Temperatur bzw. des Zeitablaufes möglich.

    Regler
    Die Steuerung der Heizung erfolgt über eine Gradientenregelung. Die Steigungsgeschwindigkeit des Temperaturanstiegs dT/dt bestimmt die Ein- und Ausschaltzeitpunkte. Sinnvoll für
    Einkocher ist hier ein Gradientenfaktor von "1". 

    Sensorfehler
    Anzeige -127  => VCC (Versorgungsspannung des Sensors fehlt
    Anzeige 85.00   => Byte interner Sensorfehler ggf. Leitung zu lang
    Anzeige 0.00  => Datenleitung oder GND fehlt


    Schaltvorgänge des Ein- und Ausschalten sind immer nach einer einstellbaren Wartezeit möglich. Die Wartezeit für SSR auf "0" gesetzt werden.Dadurch soll stetiges Schalten verhindert werden.

    Die Kühlfunktion ist recht einfach gehalten. Beim Erreichen der Zieltemperatur schaltet der Kompressor aus und nach einer vorwählbaren Zeit und beim Erreichen der Zieltemperatur von unten wieder ein.
    Der Rührer kann auf Dauerbetrieb oder Intervallbetrieb eingestellt werden.Vorgabe jeweils 60s.
    Das Kochen des Suds kann erst nach einer vorgegebenen Temperaturschwelle erfolgen (Vorgabe 97°C).
    Es können max. 6 Hopfengaben festgelegt werden. Erinnert wird durch den Brauerruf.


    Die Funktionen die auf den Vergleich Sollwert mit Istwert
    beruhen werden immer erst nach 10 Messungen ausgeführt.

    Anschlüsse:
    PIN 2 : Encoder => Pullup geschaltet => Encoderzuleitung mit GND versorgen
    PIN 3 : Encoder => Pullup geschaltet => Encoderzuleitung mit GND versorgen
    PIN 4 : Taster von Encoder => Pullup geschaltet => Zuleitung mit GND versorgen
    PIN 5 : DS1820 Sensor => 4,7k zw. 5V und PIN5
    PIN 6 : Heizung Phase
    PIN 7 : Heizung Nullleiter
    PIN 10: Ruf (Summer)
    PIN A1 und A2: Rührwerk
    PIN A4: Anschluss SDA vom Display
    PIN A5: Anschluss SCL vom Display


    Speicheraufteilung EEPROM ----------------------------------------------

    Rührer Dauerbetrieb J/N [19]
    Rührer Intervall EIN [20]
    Rührer Intervall AUS [21]
    Kochschwelle [22];
    Logview [30]
    Kühlzeitverzögerung [82]
    Wiedereinschaltzeit Heizung [86]
    Gradientenfaktor (kh) [90]
    Gradientenfaktor (kk) [94]
    Kühlmodus_aktiv [98]
    gespeicherte Kühltemperatur [99]
*/

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//Relaislogik festlegen

//Heizungsrelais +++++++++++++++++++
//
//Ist H_AUS als HIGH definiert, dann schltet das Relais die Heizung im ANGEZOGENEN ZUSTAND AUS (Historische Voreinstellung)
//Ist H_EIN als HIGH definiert, dann schaltet das Relais die Heizung im ANGEZOGENEN ZUSTAND EIN
//Ist R_AUS als HIGH definiert, dann schltet das Relais den Rührer im ANGEZOGENEN ZUSTAND AUS
//Ist R_EIN als HIGH definiert, dann schaltet das Relais den Rührer im ANGEZOGENEN ZUSTAND EIN
//
//Die gewünschte Logik auskommentieren und die jeweils andere einkommentieren !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

#define H_AUS HIGH
#define H_EIN LOW

//Relaislogik umkehren

// #define H_AUS LOW
// #define H_EIN HIGH

//Rührerrelais +++++++++++++++++++

#define R_AUS LOW
#define R_EIN HIGH

//Relaislogik umkehren

// #define R_AUS HIGH
// #define R_EIN LOW

//Rufrelais +++++++++++++++++++

#define B_AUS LOW
#define B_EIN HIGH

//Relaislogik umkehren

// #define B_AUS HIGH
// #define B_EIN LOW



//LCD Initialisierung --------------------------------------------------------

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
// set the LCD address to 0x27 for a 20 chars 4 line display
// Set the pins on the I2C chip used for LCD connections:
//                    addr, en,rw,rs,d4,d5,d6,d7,bl,blpol
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // Set the LCD I2C address

//LiquidCrystal_I2C lcd(0x27,20,4);  // Set the LCD I2C address

//Click Encoder-----------------------------------------------------

#include <ClickEncoder.h>
#include <TimerOne.h>

ClickEncoder *encoder;

void timerIsr() {
  encoder->service();
}

int last;
int wert;
bool ButtonPressed = false;
byte drehen = 0;            //Drehgeber Werte



//Temperatursensor DS1820 Initialisierung ---------------------------
#include <OneWire.h>
#include <DallasTemperature.h>

// Data wire is plugged into port 5 on the Arduino
#define ONE_WIRE_BUS 5

// Setup a oneWire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);

// Pass our oneWire reference to Dallas Temperature.
DallasTemperature sensors(&oneWire);

// arrays to hold device address
DeviceAddress insideThermometer;

//Zeitmessung----------------------------------------------------------
#include <Time.h>
//---------------------------------------------------------------------

//EEPROM für Voreinstellungen----------------------------------------------------------
#include <EEPROM.h> 
//------------------------------------------------------------------

//Ausgänge an DigitalPIN --------------------------------------------
#define schalterH1  6                                   //Zuordnung Heizung Relais1
#define schalterH2  7                                   //Zuordnung Heizung Relais1
#define schalterB  10                                   //Zuordnung Braumeisterruf
//-----------------------------------------------------------------

//Es werden 2 Relais angesteuert, alternativ für einen Rührer 230V und einen für 12 V (Scheibenwischer)
//man kann natürlich auch nur einen benutzen
//Ausgänge am AnalogPIN--------------------------------------------
#define schalterR230  A1                                  //Zuordnung Rührwerk 230V
#define schalterR12   A2                                   //Zuordnung Rührwerk 12V
//-----------------------------------------------------------------


//Encoder Initialisierung ----------------------------------------------

//Encoder drehen
#define pinA 3  //können auch beliebige andere Anschlüsse sein!
#define pinB 2  //traditionell noch die Belegung von fg100

//#define ENC_DECODER (1 << 2)   //auskommentieren, wenn Probleme mit Encoder

//#define ENC_HALFSTEP 1     //auskommentieren, wenn Probleme mit Encoder

#define stepsPerNotch 4 // Klicks bis zum nächsten Schritt

// Taster drücken
#define taster 4     // Taster am Pin 4 für Bestätigung und Abbruch



// Messergebnisse -----------------------------------------------------
byte val = 0;        // Variable um Messergebnis zu speichern
//---------------------------------------------------------------------


//Allgemein Initialisierung--------------------------------------------
byte anfang = 0;
float altsekunden;
byte regelung = 0;
byte heizung = 0;
byte sensorfehler = 0;

boolean sollwerterreicht = false;
float wartezeit = -60000;
long kwarten = 0; //Wartezeit kühlen
float sensorwert;
float isttemp = 20;                                   //Vorgabe 20 damit Sensorfehler am Anfang nicht anspricht
int modus = 0;
int rufmodus = 0;
float rufsignalzeit = 0;
boolean nachgussruf = false;                          //Signal wenn Nachgusstemp erreicht
byte x = 1;                                            //aktuelle Rast Nummer
byte h = 0;                                            //aktuelle Hopfengabe
byte y = 1;                                            //Übergabewert von x für Braumeisterruf
byte n = 0;                                            //Counter Messungserhöhung zur Fehlervermeidung
byte pause = 0;                                        //Counter für Ruftonanzahl

byte sekunden = 0;                                       //Zeitzählung
byte minuten = 0;                                        //Zeitzählung
byte minutenwert = 0;                                    //Zeitzählung
byte stunden = 0;                                        //Zeitzählung

byte blinker;                                         //Hilfsvariable blinken;
byte abbruchkochen;                                   //Zeitzählung bis Kochabbruch


//Vorgabewerte zur ersten Einstellung-------------------------------------------
byte sollwert = 20;                                  //Sollwertvorgabe für Anzeige
byte maischtemp = 45;                                  //Vorgabe Einmasichtemperatur
byte rasten = 3;                                       //Vorgabe Anzahl Rasten
byte rastTemp[] = {
  0, 55, 64, 72, 72, 72
};        //Rasttemperatur Werte
byte rastZeit[] = {
  0, 15, 40, 25, 20, 20
};              //Rastzeit Werte
byte braumeister[] = {
  0, 0, 0, 0, 0, 0
};               //Braumeisterruf standart AUS
byte endtemp = 78;                                   //Vorgabewert Endtemperatur
byte kochzeit = 90;


/* Hopfengaben----------------------------------------------------------------
  "maxrasten" kann geändert werden. die Vorgabewerte in den Klammern müssen nicht, können
  aber angepasst werden. "hanz" ist die Voreinstellung für die Auswahl der Rastanzahl.
*/

byte const maxgaben = 6;
byte hopfengabe[maxgaben] = {0, 0, 0, 0, 0, 0}; //Zeiten für die Hopfengaben 90 min. können vorgegeben werden
//byte hopfengabe[maxgaben] = (90,20,10,0,0,0];
byte hanz = 3; //Vorgabewert

// Erweiterung Rührwerk-------------------------------------------------------

char ja_nein[2] = {'N', 'J'}; //Antworten Dauerbetrieb Ja/nein
byte intervalEin = 60;  //Voreinstellung 60s EIN
byte intervalAus = 60; // dann 60s AUS
char dauerbetrieb = 'N'; //Voreinstellung Intervallbetrieb
byte altdrehen;
byte tempdrehen;
bool firstinterval = true;
long intervalstart;
byte rwerk = 0;

//Kochen---------------------------------------------------------------------
bool schwelle_erreicht = false;

byte kschwelle = 97;  //Voreinstellung - ab hier ertönt der Brauerruf


unsigned long kzeitverz = 0; //Wartezeit Einschalten Kompressor NICHT ÄNDERN
long esvheizen = 0; //Einschaltverzögerung für Zündung - wird noch initialisiert

bool first = true;  //erster Anlauf ohne Zeitverzögerung
bool dclick; //Doppelclick gedrückt ?

long hwarten =  0;  //Wartezeitmessung Start

//Steigungsberechnung K/m -> Kelvin/Minute - Hinweis auf die Aufheizrate des Kessels

float alte_zeit; //Zeit bei t-1
float alte_temperatur; //Temperatur bei t-1
unsigned long messzyklus = 20000; // Zeit im ms für dT und dt Vorgabe 20s.

float gradient; // Steigungswert
float kh = 1; //Gradientenfaktor kh (Heizen) - Vorgabe 1
float kk = 1; //Gradientenfaktor kk (Kühlen) - Vorgabe 1

bool headersend = false;  //Daten Soll/Ist an LogView übertragen zunächst deaktiviert

byte altmodus;  //merkt sich den Modus für Doppelclick
byte zeitz = true;  //Zeitzählung nach Doppelclick nicht stoppen (zeitz = false)
bool knot;   //Kühlmodus nach Notfall gespeichert ?


//setup=============================================================
void setup()
{
  Serial.begin(9600);  // Serielle Schnittstelle für LogView

  drehen = sollwert;

  encoder = new ClickEncoder(pinB, pinA, taster, stepsPerNotch);

  Timer1.initialize(1000);
  Timer1.attachInterrupt(timerIsr); 

  last = -1;

  pinMode(schalterH1, OUTPUT);   // DigiPin als Ausgang
  pinMode(schalterH2, OUTPUT);   


  pinMode(schalterR230, OUTPUT);  
  pinMode(schalterR12, OUTPUT);  


  digitalWrite(schalterH1, H_AUS);   // ausschalten
  digitalWrite(schalterH2, H_AUS);   // ausschalten
 

  pinMode(schalterB, OUTPUT); 


  //LCD Setup ------------------------------------------------------
  lcd.begin(20, 4); 

  lcd.setCursor(0, 0);
  lcd.print("AiO_UNO_20x4_V3_05R");

  lcd.setCursor(0, 2);
  lcd.print ("Allzeit Gut Sud");

  digitalWrite(schalterB, B_EIN);   // Alarmtest
  delay(2000);
  digitalWrite(schalterB, B_AUS);   // Alarm ausschalten


  x = 1;

  lcd.clear();
  // ------------------------------------------------


  //Temperatursensor DS1820 ----------------------------------------
  sensors.getAddress(insideThermometer, 0);
  // set the resolution to 9 bit (Each Dallas/Maxim device is capable of several different resolutions)
  sensors.setResolution(insideThermometer, 12);
  //---------------------------------------------------------------

  //Rührparameter einlesen -------------------------------------------

  if (EEPROM.read(19) == 0) dauerbetrieb = 'N';
  if (EEPROM.read(19) == 1) dauerbetrieb = 'J';

  intervalEin = EEPROM.read(20);
  intervalAus = EEPROM.read(21);

  if (intervalEin == 255) (intervalEin = 60);
  if (intervalAus == 255) (intervalAus = 60);

  // Kochschwelle einlesen -----------------------------------------------
  kschwelle = EEPROM.read(22);

  if (kschwelle == 255) (kschwelle = 97);
  //----------------------------------------------------------------------

  //Wiedereinschaltverzögerung Heizungsrelais ----------------------------

  if (EEPROMReadlong(86) > 100000) EEPROMWritelong(86, 10000);
  esvheizen = EEPROMReadlong(86);

  //Wiedereinschaltverzögerung Kühlung ----------------------------

  if (EEPROMReadlong(82) > 300000) EEPROMWritelong(82, 300000);
  kzeitverz = EEPROMReadlong(82);

  if (EEPROMReadlong(90) > 100) EEPROMWritelong(90, kh * 10); //Vorgabewert Gradientenfaktor für Heizen
  kh = EEPROMReadlong(90) / 10;

  if (EEPROMReadlong(94) > 100) EEPROMWritelong(94, kk * 10); //Vorgabewert Gradientenfaktor für Heizen
  kk = EEPROMReadlong(94) / 10;

  //Logview verwenden ?

  headersend = EEPROM.read(30);


  //----------------------------------------------------------------------------


  //Rührer Aus-------------------------------------------
  rwerk = 1;
  ruehrerEinAus(0); //zunächst natürlich AUS


  //für Gradientenberechnung ---------------------------------------------------

  alte_zeit = millis();

  alte_temperatur = 20;  // geschätzter Vorgabewert - wird nach 20s überschrieben

  if (EEPROM.read(98) == 1) {modus = 1; knot = false; drehen = EEPROM.read(99);} else knot = true; //Stromausfall im Kühlmodus ?
}


//loop=============================================================
void loop()
{

  drehen += encoder->getValue();
  
  if (drehen != last) {
    last = drehen;
  }

   ButtonPressed = false;
   ClickEncoder::Button b = encoder->getButton();
  if (b != ClickEncoder::Open)
  {
    switch (b)
    {
      case ClickEncoder::Pressed:
        break;
      case ClickEncoder::Held:
        altmodus = modus;
        modus = 80;
        break;
      case ClickEncoder::Released:
         ButtonPressed = false;
        break;
      case ClickEncoder::Clicked:
        ButtonPressed = true;
        break;
      case ClickEncoder::DoubleClicked:
        altmodus = modus; //Merker für letzten Modus
        tempdrehen = drehen; //Merker für aktuellen Wert von drehen
        anfang = 0;
        dclick = true;  //Doppelclick wurde betätigt  
        modus = 5;
        zeitz = false;
        break;;
    }
  }

  if (headersend) print_logview();

  // Zeitermittlung ------------------------------------------------------
  sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
  minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
  stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung
  //---------------------------------------------------------------


  // Temperatursensor DS1820 Temperaturabfrage ---------------------
  // call sensors.requestTemperatures() to issue a global temperature
  sensors.requestTemperatures(); // Send the command to get temperatures
  sensorwert = sensors.getTempC(insideThermometer);
  if ((sensorwert != isttemp) && (n < 10)) // Messfehlervermeidung
  { // des Sensorwertes
    n++;                                   // nach mehreren
  }                                      // Messungen
  else
  {
    isttemp = sensorwert;
    n = 0;
  }
  //---------------------------------------------------------------


  // Sensorfehler -------------------------------------------------
  // Sensorfehler -127 => VCC fehlt
  // Sensorfehler 85.00 => interner Sensorfehler ggf. Leitung zu lang
  //                       nicht aktiviert
  // Sensorfehler 0.00 => Datenleitung oder GND fehlt

  if (regelung == 1) //nur bei Modi mit Heizung
  {
    if ((int)isttemp == -127 || (int)isttemp == 0 ) {
      //zur besseren Erkennung Umwandling in (int)-Wert
      //sonst Probleme mit der Erkennung gerade bei 0.00
      if (sensorfehler == 0)
      {
        rufmodus = modus;
        lcd.clear();
        lcd.setCursor(8, 0);
        lcd.print("Sensorfehler");
        regelung = 0;
        heizung = 0;
        rwerk = 1;
        ruehrerEinAus(0);

        sensorfehler = 1;
        modus = 31;
      }
    } else sensorfehler = 0;
  }
  //-------------------------------------------------------------------


  // Temperaturanzeige Istwert ---------------------------------------
  lcd.setCursor(9, 3);
  lcd.print("ist ");
  lcd.print(float(sensorwert), 2);
  if (sensorwert > 10)
  {
    lcd.print(char(0xDF));
    lcd.print("C");
  }
  else
  {
    lcd.print(char(0xDF));
    lcd.print("C ");
  }
  //-------------------------------------------------------------------



   //Heizregelung----------------------------------------------------
  if (regelung == 1)
  {
    // Temperaturanzeige Sollwert ---------------------------------------
    lcd.setCursor(16, 1);
    lcd.print(int(sollwert), 1);
    lcd.print(char(0xDF)); lcd.print("C ");

    //-------------------------------------------------------------------

    if ((modus == 27) || (modus == 28) || (modus == 29) || (modus == 30) || (modus == 2) || (modus == 1))  //Maischeautomatik/Nachguss/Kühlen
    {
      lcd.setCursor(0, 0);
      if ((modus != 2) && (modus != 1)) lcd.print("Maischen");
      if ((alte_zeit + messzyklus) < millis()) {
        gradient = (isttemp - alte_temperatur) / (((millis() - alte_zeit) / messzyklus) * 60000 / messzyklus);

        alte_zeit = millis();
        alte_temperatur = isttemp;
      }
      if (modus != 2) {
        lcd.setCursor(0, 1);
        lcd.print(gradient);
        lcd.print(" K/m  ");
      }

    }

    //Gradientenregelung

    if (modus != 1) { //Heizungsregelung
      if (isttemp > (sollwert - (gradient * kh))) {
        if (heizung == 1) {
          heizung = 0;
          hwarten = millis();
          sollwerterreicht = true;
        }

      }
      if (isttemp <= (sollwert - (gradient * kh))) {
        if ((millis() > (hwarten + esvheizen)) || first) {
          first = false;
          if (isttemp <= sollwert) heizung = 1;
        }
        sollwerterreicht = false;
      }
    }

    if (modus == 1) {  //Kühlregelung
      if (isttemp < (sollwert - (gradient * kk))) {
        if (heizung == 1) {
          heizung = 0;
          hwarten = millis();
        }

      }
      if (isttemp >= (sollwert - (gradient * kk))) {
        if ((millis() > (hwarten + kzeitverz)) || first) {
          first = false;
          if (isttemp >= sollwert) heizung = 1;
        }
      }
    }
  }


  // Zeigt "Heizen" wenn Heizen und "Kühlen wenn Kühlen --------------------------------
  if (heizung == 1)
  {
    lcd.setCursor(0, 3);
    if (modus != 1) {lcd.print("Heizen");} else  {
      lcd.print("K");
      lcd.print(char(0xF5));
      lcd.print("hlen");
    }
    digitalWrite(schalterH1, H_EIN);   // einschalten
    digitalWrite(schalterH2, H_EIN);   // einschalten


    if (modus != 1) if (rwerk == 0) ruehrerEinAus(1);  //nicht im Kühlmodus
  }
  else
  {
    lcd.setCursor(0, 3); lcd.print("       ");
    digitalWrite(schalterH1, H_AUS);   // ausschalten
    digitalWrite(schalterH2, H_AUS);   // ausschalten


    if (modus != 1)
    { //nicht im Kühlmodus
      if (dauerbetrieb == 'J') {
        rwerk = 0; // alter Status des Rührers
        // damit nicht fortwährend Daten gesendet werden!
        ruehrerEinAus(1)
        ;
      } //Rührer im Dauerbetrieb
      else if ((heizung == 0) && (regelung == 1) && (modus != 2) && (sollwerterreicht)) intervallbetrieb(); //Rührer im Intervallbetrieb
    }
  }
  //Ende Heizregelung---------------------------------------------------


  // Abfrage Modus

  encoder->setAccelerationEnabled(true);
  switch (modus) {
    case 0: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Hauptmenü
        altmodus = 0;
        funktion_hauptschirm();
        break;
      }
    case 1: {
        encoder->setAccelerationEnabled(true);
        regelung = 1;  //Kühlen
        altmodus = 1;
        funktion_temperatur();
        break;
      }
    case 2: {
        encoder->setAccelerationEnabled(true);
        regelung = 1;  //Nachguss
        funktion_temperatur();
        break;
      }
    case 5: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Einstellungen
        funktion_SetupMenu();
        break;
      }

    case 11: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Gradientenfaktor Heizen
        rufmodus = 0;
        khwert();
        break;
      }

    case 12: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Wartezeit Wiedereinschalten Heizungsrelais
        ESVHeiz();
        break;
      }  

    case 13: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Gradientenfaktor Kühlen
        rufmodus = 0;
        kkwert();
        break;
      }  

   case 14: {
        encoder->setAccelerationEnabled(false); 
        regelung = 0;  //Wartezeit bein Kühlen vor dem Wiedereinschalten
        wartenKuehlen();
        break;
      }    

    case 19: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Anzahl Rasten
        funktion_rastanzahl();
        break;
      }
    case 20: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Eingabe Einmaischtemperatur
        funktion_maischtemperatur();
        break;
      }
    case 21: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Eingabe der Temperatur der Rasten
        funktion_rasteingabe();
        break;
      }
    case 22: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Eingabe der Rastzeitwerte
        funktion_zeiteingabe();
        break;
      }
    case 23: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Rührwerk Dauerlauf?
        ruehrwerk();
        break;
      }
    case 231: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Rührwerk Intervall EIN
        ruehrwerk_ein();
        break;
      }
    case 232: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //Rührwerk Intervall AUS
        ruehrwerk_aus();
        break;
      }

    case 239: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Kochschwelle - ab hier Brauerruf
        kochschwelle();
        break;
      }
    case 24: {
        regelung = 0;  //Eingabe Braumeisterruf An/Aus
        funktion_braumeister();
        break;
      }
    case 25: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;  //Abmaischtemperatur
        funktion_endtempeingabe();
        break;
      }
    case 26: {
        regelung = 0;  //Start Maischeprozess
        funktion_startabfrage();
        break;
      }
    case 27: {
        encoder->setAccelerationEnabled(true);
        regelung = 1;  //Automatischer Ablauf Maischen
        funktion_maischtemperaturautomatik();
//        headersend = true;
        break;
      }
    case 28: {
        encoder->setAccelerationEnabled(true); 
        regelung = 1;
        funktion_tempautomatik();
        break;
      }
    case 29: {
        encoder->setAccelerationEnabled(true);
        regelung = 1;
        funktion_zeitautomatik();
        break;
      }
    case 30: {
        encoder->setAccelerationEnabled(true); 
        regelung = 1;
        funktion_endtempautomatik();
        break;
      }
    case 31: {
        regelung = 0;
        funktion_braumeisterrufalarm();
        break;
      }
    case 32: {
        regelung = 0;
        funktion_braumeisterruf();
        break;
      }
    case 38: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;
        anzahl_hopfengaben();
        break;
      }
    case 39: {
        regelung = 0;
        funktion_hopfengabe();
        break;
      }
    case 40: {
        encoder->setAccelerationEnabled(true);
        regelung = 0;
        funktion_kochzeit();
        break;
      }
    case 41: {
        regelung = 0;
        funktion_startkochen();
        break;
      }
    case 42: {
        regelung = 0;
        funktion_kochen();
        break;
      }

    case 43: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;  //LogViewEinAus
        LogviewEinAus();
        break;
      }
    case 80: {
        encoder->setAccelerationEnabled(false);
        regelung = 0;
        funktion_abbruch();
        break;
      }
  }
}
// Ende Loop
// ------------------------------------------------------------------


//Funktionen========================================================
// ab hier Funktionen

// Funktion Hauptschirm---------------------------------------------
void funktion_hauptschirm()      //Modus=0
{

  if ((anfang == 0) || (dclick))

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Maischen");
    lcd.setCursor(2, 1);
    lcd.print("Nachguss");
    lcd.setCursor(2, 2);
    lcd.print("Kochen");
    lcd.setCursor(14, 0);
    lcd.print("K"); lcd.print(char(0xF5)); lcd.print("hlen");
    lcd.setCursor(14, 1);
    lcd.print("Setup");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
    dclick = false;
  }


  if (drehen == 255)
  {
    drehen = 4;
  }
  if (drehen == 5)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(12, 0);
    lcd.print("  ");
    lcd.setCursor(12, 1);
    lcd.print("  ");
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = 19;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = 2;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = 40;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = 1;
    lcd.setCursor(12, 0);
    lcd.print("=>");
  }

  if (drehen == 4)
  {
    altdrehen = 5;
    rufmodus = 5;
    lcd.setCursor(12, 1);
    lcd.print("=>");
  }
  
    if (ButtonPressed)
    {
      modus = rufmodus;
      if (modus == 1)
      { //Übergabe an Modus1
        drehen = EEPROM.read(99); //ganzzahliger Vorgabewert für Kühlen 
        EEPROM.write(98, 1); //Kühlmodus bei Stromausfall aktiv
//        knot = true;
        altmodus = 1;
      }
      if (modus == 2)
      { //Übergabe an Modus2
        drehen = 78;                    //Nachgusstemperatur Sollwert
      }                             
      anfang = 0;
      lcd.clear();
    }

}
//-----------------------------------------------------------------

//Funktion SetupMenu-----------------------------------------------

void funktion_SetupMenu()      //Modus=5
{
  if (anfang == 0)

  {
    lcd.clear();
    lcd.setCursor(2, 0);
    lcd.print("Schwelle");
    lcd.setCursor(2, 1);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrer");
    lcd.setCursor(2, 2);
    lcd.print("Regler");
    lcd.setCursor(13, 0);
    lcd.print("LogView");
    lcd.setCursor(13, 1);
    lcd.print("Zur"); lcd.print(char(0xF5)); lcd.print("ck");
    drehen = 0;
    altdrehen = 0;
    anfang = 1;
  }


  if (drehen == 255)
  {
    drehen = 4;
  }
  if (drehen == 5)
  {
    drehen = 0;
  }

  if (drehen != altdrehen)
  {
    lcd.setCursor(0, 0);
    lcd.print("  ");
    lcd.setCursor(0, 1);
    lcd.print("  ");
    lcd.setCursor(0, 2);
    lcd.print("  ");
    lcd.setCursor(11, 0);
    lcd.print("  ");
    lcd.setCursor(11, 1);
    lcd.print("  ");
    altdrehen = drehen;
  }

  if (drehen == 0)
  {
    altdrehen = 0;
    rufmodus = 239;
    lcd.setCursor(0, 0);
    lcd.print("=>");
  }

  if (drehen == 1)
  {
    altdrehen = 1;
    rufmodus = 23;
    lcd.setCursor(0, 1);
    lcd.print("=>");
  }

  if (drehen == 2)
  {
    altdrehen = 2;
    rufmodus = 11;
    lcd.setCursor(0, 2);
    lcd.print("=>");
  }

  if (drehen == 3)
  {
    altdrehen = 3;
    rufmodus = 43;
    lcd.setCursor(11, 0);
    lcd.print("=>");
  }

  if (drehen == 4)
  {
    altdrehen = 4;
    rufmodus = altmodus;
    lcd.setCursor(11, 1);
    lcd.print("=>");
   
  }

  if (ButtonPressed)
  {

    if (dclick) {
      anfang = 1;
    } else {
      anfang = 0;
    }
    modus = rufmodus;
    drehen = tempdrehen; 
    first = true;
    lcd.clear();
  }

}


// Funktion nur Temperaturregelung---------------------------------
void funktion_temperatur()      //Modus=1 bzw.2
{

  if (drehen == 255) drehen = 0;
  sollwert = drehen;


  if (modus == 1)
  {
    lcd.setCursor(0, 0);
    lcd.print("K"); lcd.print(char(0xF5)); lcd.print("hlen");
  }

  if (modus == 2)
  {
    lcd.setCursor(0, 0);
    lcd.print("Nachguss");
    
  }

  if ((modus == 2) && (isttemp >= sollwert) && (nachgussruf == false))   //Nachgussruf -> Sollwert erreicht
  {
    nachgussruf = true;
    rufmodus = modus;          //Rufalarm
    modus = 31;
    y = 0;
    braumeister[y] = 2;        //nur Ruf und weiter mit Regelung
  }

  lcd.setCursor(11, 1);
  lcd.print("soll");

}
//-----------------------------------------------------------------


// Funktion Eingabe der Rastanzahl------------------------------------
void funktion_rastanzahl()          //Modus=19
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    lcd.setCursor(12, 1);
    lcd.print("Rasten");

    drehen = rasten;
    anfang = 1;
    dclick = false;
  }

  //Vorgabewerte bei verschiedenen Rasten
  if (rasten != drehen)
  {
    if ((int)drehen == 1)
    {
      rastTemp[1] = 67;
      rastZeit[1] = 60;
      maischtemp = 65;
    }
    if (drehen == 2)
    {
      rastTemp[1] = 62;
      rastZeit[1] = 30;
      rastTemp[2] = 72;
      rastZeit[2] = 35;
      maischtemp = 62;
    }
    if (drehen == 3)
    {
      rastTemp[1] = 55;
      rastZeit[1] = 10;
      rastTemp[2] = 62;
      rastZeit[2] = 35;
      rastTemp[3] = 72;
      rastZeit[3] = 25;
      maischtemp = 55;
    }
    if (drehen == 4)
    {
      rastTemp[1] = 40;
      rastZeit[1] = 20;
      rastTemp[2] = 55;
      rastZeit[2] = 15;
      rastTemp[3] = 64;
      rastZeit[3] = 35;
      rastTemp[4] = 72;
      rastZeit[4] = 25;
      maischtemp = 35;
    }
    if (drehen == 5)
    {
      rastTemp[1] = 35;
      rastZeit[1] = 20;
      rastTemp[2] = 40;
      rastZeit[2] = 20;
      rastTemp[3] = 55;
      rastZeit[3] = 15;
      rastTemp[4] = 64;
      rastZeit[4] = 35;
      rastTemp[5] = 72;
      rastZeit[5] = 25;
      maischtemp = 30;
    }
  }

  rasten = drehen;


  if (rasten <= 1)
  {
    rasten = 1;
    drehen = 1;
  }
  if (rasten >= 5)
  {
    rasten = 5;
    drehen = 5;
  }

  lcd.setCursor(19, 1);
  lcd.print(rasten);

    if (ButtonPressed)
    {
      modus++;
      anfang = 0;
    }
}
//------------------------------------------------------------------


// Funktion Maischtemperatur-----------------------------------------
void funktion_maischtemperatur()      //Modus=20
{

  if ((anfang == 0) || (dclick))
  {
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = maischtemp;
    first = true;
    anfang = 1;
    dclick = false;
  }

  maischtemp = drehen;

  if (maischtemp <= 10)
    maischtemp = 10;

  if (maischtemp >= 105)
    maischtemp = 105;



  lcd.setCursor(0, 1);
  lcd.print("Einmaischen         ");
  lcd.setCursor(16, 1);
  lcd.print(maischtemp);
  lcd.print(char(0xDF)); lcd.print("C ");

    if (ButtonPressed)
    {
      modus++;
      anfang = 0;
    }
}
//------------------------------------------------------------------


// Funktion Rasteingabe Temperatur----------------------------------
void funktion_rasteingabe()      //Modus=21
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = rastTemp[x];
    anfang = 1;
    dclick = false;
  }

  rastTemp[x] = drehen;

  if (rastTemp[x] <= 10)
  {
    rastTemp[x] = 9;
    drehen = 9;
  }
  if (rastTemp[x] >= 105)
  {
    rastTemp[x] = 105;
    drehen = 105;
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");
  lcd.setCursor(16, 1);
  lcd.print(int(rastTemp[x]));
  lcd.print(char(0xDF)); lcd.print("C ");

    if (ButtonPressed)
    {
      modus++;
      anfang = 0;
    }
}
//------------------------------------------------------------------


// Funktion Rasteingabe Zeit----------------------------------------
void funktion_zeiteingabe()      //Modus=22
{

  if ((anfang == 0) || (dclick))
  {
    drehen = rastZeit[x];
    anfang = 1;

    lcd.setCursor(0, 0);
    lcd.print("Eingabe");

    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");
    lcd.setCursor(15, 1);
    lcd.print(int(rastTemp[x]));
    lcd.print(char(0xDF)); lcd.print("C ");


    dclick = false;
  }

  rastZeit[x] = drehen;

  if (rastZeit[x] <= 1)
  {
    rastZeit[x] = 1;
    drehen = 1;
  }
  if (rastZeit[x] >= 99)
  {
    rastZeit[x] = 99;
    drehen = 99;
  }

  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  if (ButtonPressed)
  {
    modus = 24;
    anfang = 0;
  }
}
//------------------------------------------------------------------

//------------------------------------------------------------------
void ruehrwerk() //Modus=23
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Dauerbetrieb");
    drehen = EEPROM.read(19);
    anfang = 1;
    dclick = false;
  }

  char antwort = ja_nein[drehen % 2];

  lcd.setCursor(14, 1); lcd.print(antwort);

  dauerbetrieb = antwort;

    if (ButtonPressed)
    {
      if (dauerbetrieb == 'N') {
        modus = 231;
        EEPROM.write(19, 0);
      } else {
        modus = 5;
        EEPROM.write(19, 1);
      }
      speichern();
      anfang = 0;
    }
}

// Funktion Rührwerk Intervall EIN---------------------------------------------
void ruehrwerk_ein()      //Modus=231
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalEin;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;
  intervalEin = drehen;

  if (intervalEin > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall EIN: "); lcd.print(intervalEin); lcd.print("s   ");
  }

    if (ButtonPressed)
    {
      EEPROM.write(20, intervalEin);
      speichern();
      modus++;
      anfang = 0;
    }
}

//-----------------------------------------------------------------------------

// Funktion Rührwerk Intervall AUS---------------------------------------------

void ruehrwerk_aus()      //Modus=232
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("R"); lcd.print(char(0xF5)); lcd.print("hrwerk");
    lcd.setCursor(0, 1);
    lcd.print("Intervall");
    drehen = intervalAus;
    anfang = 1;
    dclick = false;
  }

  if (drehen > 250) drehen = 0;

  intervalAus = drehen;

  if (intervalAus > 9)
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s  ");
  }
  else
  {
    lcd.setCursor(0, 1); lcd.print("Intervall AUS: "); lcd.print(intervalAus); lcd.print("s   ");
  }

    if (ButtonPressed)
    {
      EEPROM.write(21, intervalAus);
      speichern();
      modus = 5;
      anfang = 0;
    }
}


// Funktion Kochschwelle einstellen ---------------------------------------------

void kochschwelle()  //Modus 239
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Kochschwelle");
    lcd.setCursor(0, 1); lcd.print("Ruf bei:");
    drehen = EEPROM.read(22); //EPROM-Speicher auslesen
    anfang = 1;
    dclick = false;
  }
  else
  {


    if (drehen > 99) drehen = 20;
    if (drehen <= 20) drehen = 99;

    kschwelle = drehen;


    lcd.setCursor(9, 1); lcd.print(kschwelle);


    if (ButtonPressed)
    {
      EEPROM.write(22, kschwelle);
      speichern();
      modus = 5;
      anfang = 0;
    }
  }
}
// -----------------------------------------------------------------



// Funktion Intervallbetrieb---------------------------------------
void intervallbetrieb()
{
  if (regelung == 1)
  {
    unsigned long ein = 1000 * (unsigned long)intervalEin;
    unsigned long aus = 1000 * (unsigned long)intervalAus;

    if (firstinterval)
    {
      intervalstart = millis(); firstinterval = false;
    }
    else if ((intervalstart  + ein)  >= millis()) {
      if (rwerk == 0) ruehrerEinAus(1);
    }
    else if ((intervalstart  + ein + aus) >= millis()) {
      if (rwerk == 1) ruehrerEinAus(0);
    }
    else firstinterval = true;
  }
}
//------------------------------------------------------------------

void ruehrerEinAus(int ruehrer)
{
  if (modus == 31) // bei Ruf Nachgußbereitung den Rührer nicht einschalten
  {

    digitalWrite(schalterR230, R_AUS);
    digitalWrite(schalterR12, R_AUS);
  }
  else if ((regelung == 1) && (modus != 2)) // außer bei Nachgußbereitung

    if (ruehrer == 1)
    {
      if (rwerk == 0)
      {
        // Rührer wird eingeschaltet
        digitalWrite(schalterR230, R_EIN);
        digitalWrite(schalterR12, R_EIN);
        rwerk = 1;
      }


    }
  if (ruehrer == 0)
  {
    if (rwerk == 1)
    {
      //Rührer wird ausgeschaltet
      digitalWrite(schalterR230, R_AUS);
      digitalWrite(schalterR12, R_AUS);
      rwerk = 0;
    }
  }
}


// Funktion Braumeister---------------------------------------------
void funktion_braumeister() //Modus=24
{

  braumeister[x] = 0;
  if (x < rasten)
  {
    x++;
    modus = 21;           //Sprung zur Rasttemperatureingabe
    anfang = 0;
  }
  else
  {
    x = 1;
    modus++;             //Sprung zur Rastzeiteingabe
    anfang = 0;
  }
}
//------------------------------------------------------------------


// Funktion Ende Temperatur-----------------------------------------
void funktion_endtempeingabe()      //Modus=25
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Eingabe");
    drehen = endtemp;
    anfang = 1;
  }

  endtemp = drehen;

  if (endtemp <= 10)
    endtemp = 10;

  if (endtemp >= 80)
    endtemp = 80;

  lcd.setCursor(0, 1);
  lcd.print("Abmaischen");
  lcd.setCursor(16, 1);
  lcd.print(int(endtemp));
  lcd.print(char(0xDF)); lcd.print("C ");

 
    if (ButtonPressed)
    {
      modus++;
      anfang = 0;
    }
}
//------------------------------------------------------------------


// Funktion Startabfrage--------------------------------------------
void funktion_startabfrage()      //Modus=26
{
  if (anfang == 0)
  {
    lcd.clear();
    anfang = 1;
    altsekunden = millis();
  }

  if (millis() >= (altsekunden + 1000))
  {
    lcd.setCursor(13, 1);
    lcd.print("       ");
    if (millis() >= (altsekunden + 1500))
      altsekunden = millis();
  }
  else
  {
    lcd.setCursor(13, 1);
    lcd.print("Start ?");
  }

    if (ButtonPressed)
    {
      anfang = 0;
      modus++;
    }
}
//------------------------------------------------------------------


// Funktion Automatik Maischtemperatur---------------------------------
void funktion_maischtemperaturautomatik()      //Modus=27
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Maischen");
    drehen = maischtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
  }

  maischtemp = drehen;

  sollwert = maischtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    altmodus = modus;
    modus++;
    rufmodus = modus;
    y = 0;
    braumeister[y] = 1;
    modus = 31;
  }
}
//------------------------------------------------------------------


// Funktion Automatik Temperatur------------------------------------
void funktion_tempautomatik()      //Modus=28
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(14, 0);
    lcd.print(x);
    lcd.print(".Rast");

    firstinterval = true;  //Intervallbetrieb Rührwerk
    rwerk = 0;

    drehen = rastTemp[x];
    anfang = 1;
    first = true;
    dclick = false;
  }

  rastTemp[x] = drehen;

  sollwert = rastTemp[x];


  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    modus++;                //zur Zeitautomatik
    anfang = 0;
  }
}
//------------------------------------------------------------------


// Funktion Automatik Zeit------------------------------------------
void funktion_zeitautomatik()      //Modus=29
{
  if (anfang == 0)
  {
    drehen = rastZeit[x];              //Zuordnung für Encoder
  }

  lcd.setCursor(14, 0);
  lcd.print(x);
  lcd.print(".Rast");


  if (rastZeit[x] < 10)
  {
    lcd.setCursor(14, 2);
    lcd.print(" ");
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }
  else
  {
    lcd.setCursor(14, 2);
    lcd.print(int(rastZeit[x]));
    lcd.print(" min");
  }


  // Zeitzählung---------------
  if (anfang == 0)
  {
    if (zeitz) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    lcd.setCursor(0, 2);
    lcd.print("        ");
    anfang = 1;
  }

  lcd.setCursor(0, 2);

  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(1, 2);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }


  if (stunden == 0)
    minuten = minutenwert;
  else
    minuten = ((stunden * 60) + minutenwert);

  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(1, 2);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(1, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(0, 2);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  // Ende Zeitzählung---------------------

  rastZeit[x] = drehen;   //Encoderzuordnung

  if (minuten >= rastZeit[x])  // Sollwert erreicht ?
  {
    anfang = 0;
    y = x;
    if (x < rasten)
    {
      modus--;                // zur Temperaturregelung
      x++;                    // nächste Stufe
    }
    else
    {
      x = 1;                              //Endtemperatur
      modus++;                            //Endtemperatur
    }

    if (braumeister[y] > 0)
    {
      rufmodus = modus;
      modus = 31;
    }
  }
}
//------------------------------------------------------------------


// Funktion Automatik Endtemperatur---------------------------------
void funktion_endtempautomatik()      //Modus=30
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(10, 0);
    lcd.print("Abmaischen");

    drehen = endtemp;    //Zuordnung Encoder
    anfang = 1;
    dclick = false;
  }

  endtemp = drehen;

  sollwert = endtemp;

  if (isttemp >= sollwert)  // Sollwert erreicht ?
  {
    modus = 80;                //Abbruch
    rufmodus = modus;
    modus = 31;
    regelung = 0;              //Regelung aus
    heizung = 0;               //Heizung aus
    y = 0;
    braumeister[y] = 1;
  }
}
//------------------------------------------------------------------


// Funktion braumeisterrufalarm---------------------------------------
void funktion_braumeisterrufalarm()      //Modus=31
{
  if (anfang == 0)
  {
    rufsignalzeit = millis();
    altsekunden = millis();
    anfang = 1;
  }

  if (altmodus == 27) //Einmaischtemperatur erreicht !
  {
    altmodus = 31;
    heizung = 0;
    rwerk = 1;
    ruehrerEinAus(0);
  }

  if (millis() >= (altsekunden + 1000)) //Blinken der Anzeige und RUF
  {
    lcd.setCursor(17, 2);
    lcd.print("   ");
    digitalWrite(schalterB, B_AUS);

    if (millis() >= (altsekunden + 1500))
    {
      altsekunden = millis();
      pause++;
    }
  }
  else
  {
    lcd.setCursor(17, 2);
    lcd.print("Ruf");
    if (pause <= 4)
    {
      digitalWrite(schalterB, B_EIN);

    }
    if (pause > 8)
      pause = 0;
  }                                   //Blinken der Anzeige und RUF

  //20 Sekunden Rufsignalisierung wenn "Ruf Signal"
  if (braumeister[y] == 2 && millis() >= (rufsignalzeit + 20000))
  {
    anfang = 0;
    pause = 0;
    digitalWrite(schalterB, B_AUS);   // Alarm ausschalten

    modus = rufmodus;
    lcd.setCursor(17, 2);
    lcd.print("   ");
  }
  //weiter mit Programmablauf

    if (ButtonPressed)
    {
      pause = 0;
      anfang = 0;
      digitalWrite(schalterB, B_AUS);   // Alarm ausschalten

      if (braumeister[y] == 2)
      {
        lcd.setCursor(17, 2);
        lcd.print("   ");
        if (sensorfehler == 1) {
          sensorfehler = 0;  //Sensorfehler oder Alarmtest
          lcd.setCursor(8, 0);
          lcd.print("            ");
        }
        modus = rufmodus;
      }
      else
        modus++;
    }
}
//------------------------------------------------------------------


// Funktion braumeisterruf------------------------------------------
void funktion_braumeisterruf()      //Modus=32
{
  if (anfang == 0)
  {
    anfang = 1;
  }


  if (millis() >= (altsekunden + 1000))
  {
    lcd.setCursor(12, 2);
    lcd.print("        ");
    if (millis() >= (altsekunden + 1500))
      altsekunden = millis();
  }
  else
  {
    lcd.setCursor(0, 2); lcd.print("            ");
    lcd.setCursor(12, 2);
    lcd.print("weiter ?");
  }


 
    if (ButtonPressed)
    {

      lcd.setCursor(12, 2);
      lcd.print("        ");  //Text "weiter ?" löschen

      lcd.setCursor(0, 3);
      lcd.print("             "); //Löscht Text bei
      if (sensorfehler == 1) {
        sensorfehler = 0;  //Sensorfehler oder Alarmtest
        lcd.setCursor(8, 0);
        lcd.print("            ");
      }

      anfang = 0;
      modus = rufmodus;
      if (rufmodus == 42) {
        anfang = 1;   // Hopfen zugegeben!
        lcd.setCursor(0, 2);
        lcd.print("                    ");
      }

      delay(500);     //kurze Wartezeit, damit nicht
      //durch unbeabsichtigtes Drehen
      //der nächste Vorgabewert
      //verstellt wird
    }
}
//------------------------------------------------------------------

void anzahl_hopfengaben() //Modus=38
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengaben");
    lcd.setCursor(5, 1);
    lcd.print("Anzahl");
    anfang = 1;
    hanz = 3;
    drehen = hanz;
    lcd.setCursor(14, 1);
    lcd.print(hanz);
    dclick = false;
  }


  if (hanz < 9)
  {
    lcd.setCursor(14, 1); lcd.print(hanz);
  }
  else
  {
    lcd.setCursor(13, 1); lcd.print(hanz);
  }

  hanz = drehen;

  if (drehen >= maxgaben)
  {
    hanz = maxgaben; drehen = maxgaben;
  }

  if (drehen <  2)
  {
    hanz = 1; drehen = 1;
  }

  if (ButtonPressed)
    {
      modus = 39;
      anfang = 0;
    }


}



// Funktion Hopfengabe-------------------------------------------------
void funktion_hopfengabe()      //Modus=39
{
  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Hopfengabe:");
    lcd.setCursor(0, 1);
    lcd.print("Kochen ");
    lcd.print("f"); lcd.print(char(0xF5)); lcd.print("r");

    if (kochzeit >= 60)  // weniger ist eigentlich nicht sinnvoll
    {
      if (h == 0) hopfengabe[h] = kochzeit;  //Typische Zeiten für das Hopfenkochen
      if (h == 1) hopfengabe[h] = kochzeit - 20;
      if (h == 2) hopfengabe[h] = 10;
    }

    drehen = hopfengabe[h];

    anfang = 1;
    dclick = false;
  }
  lcd.setCursor(11,0);lcd.print(h + 1);

  hopfengabe[h] = drehen;
  lcd.setCursor(12, 1);

  lcd.print(drehen);


  lcd.print(" min ");

  if (hopfengabe[h] <= 0)
  {
    hopfengabe[h] = 0;
    drehen = 0;
  }
  if (hopfengabe[h] >= kochzeit)
  {
    hopfengabe[h] = kochzeit;
    drehen = kochzeit;
  }



    if (ButtonPressed)
    {
      if (h == (hanz - 1)) {
        lcd.clear();
        modus = 41;

      } else {
        h++;
      }
      anfang = 0;
    }
}
//------------------------------------------------------------------




// Funktion Kochzeit-------------------------------------------------
void funktion_kochzeit()      //Modus=40
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Aufheizen");
    lcd.setCursor(4, 1);
    lcd.print("Kochzeit");
    lcd.setCursor(17, 1);
    lcd.print("min");

    lcd.setCursor(2, 2);
    lcd.print("Dr"); lcd.print(char(0xF5)); lcd.print("cken = weiter");

    drehen = kochzeit;

    heizung = 1; //Heizung einschalten
    rwerk = 1;
    ruehrerEinAus(0);

    anfang = 1;
    dclick = false;
  }

  kochzeit = drehen; //5min-Sprünge

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("  ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }

    if (ButtonPressed)
    {
      modus = 38; // Anzahl Hopfengaben
      anfang = 0;
    }
}
//------------------------------------------------------------------


// Funktion Startkochzeit-------------------------------------------
void funktion_startkochen()      //Modus=41
{
  if ((anfang == 0) || (dclick))
  {   
    anfang = 1;
    altsekunden = millis();
    abbruchkochen = 60;
    blinker = 0;
    dclick = false;
  }
  lcd.setCursor(0, 0); lcd.print("Ruf bei "); lcd.print(kschwelle); lcd.print(char(0xDF)); lcd.print("C"); lcd.print("  ");
  if (millis() <= (altsekunden + 1000))
  {

    if ((isttemp >= kschwelle) || (schwelle_erreicht))
    {

      schwelle_erreicht = true;
      if (anfang != 3) {
        lcd.setCursor(0, 1);
        lcd.print("Abbruch in ");
        if (abbruchkochen <= 60) {
          lcd.print(abbruchkochen);
          lcd.print(" s ");
        }
      }
      lcd.setCursor(0, 2);
      if (blinker == 0)
      {
        if (anfang == 3)
        {
          lcd.print("Start Zeitz");
          lcd.print(char(0xE1));
          lcd.print("hlung?");
        }
        else
        {
          lcd.print("Bitte quittieren!");
        }
        if (anfang == 1) {
          signalton(500);
          anfang = 2;
        } else if (anfang == 2)
        { if (abbruchkochen > 61) modus = 80;
          if (abbruchkochen % 10 == 0) signalton(500);
        }
      }
      else lcd.print("                   ");

    }
    else
    {
      if (blinker == 0)
      {
        lcd.setCursor(0, 2);
        lcd.print("Warten!");
        schwelle_erreicht = false;
      }

      if (blinker == 1)
      {
        lcd.setCursor(0, 2);
        lcd.print("                   ");
      }
    }

  } else {
    if (blinker == 0) blinker = 1;
    else blinker = 0;
    altsekunden = millis();
    if (schwelle_erreicht) abbruchkochen--;
  }

  if (schwelle_erreicht)
  {
    if (ButtonPressed)
    {
        if (anfang == 3)
        {
          anfang = 0;
          lcd.setCursor(0, 2); lcd.print("               ");
          modus = 42;
        }
        if (anfang == 2) {
          anfang = 3;
          lcd.setCursor(0, 1);
          lcd.print("                    ");
        }

      }
  }
}
//------------------------------------------------------------------


// Funktion Kochen------------------------------------------
void funktion_kochen()      //Modus=42
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Kochen");
    lcd.setCursor(4, 1);
    lcd.print("Kochzeit");
    lcd.setCursor(17, 1);
    lcd.print("min");

    if (!dclick) setTime(00, 00, 00, 00, 01, 01); //.........Sekunden auf 0 stellen

    delay(400); //Anzeigepause

    sekunden = second();  //aktuell Sekunde abspeichern für die Zeitrechnung
    minutenwert = minute(); //aktuell Minute abspeichern für die Zeitrechnung
    stunden = hour();     //aktuell Stunde abspeichern für die Zeitrechnung

    anfang = 1;
    if (!dclick) h = 0;  //Hopfengabengabe Zähler
    drehen = kochzeit;
    dclick = false;
  }

  if (kochzeit < 100)
  {
    lcd.setCursor(12, 1);
    lcd.print("  ");
    lcd.print(kochzeit);
  }
  else
  {
    lcd.setCursor(12, 1);
    lcd.print(" ");
    lcd.print(kochzeit);
  }



  lcd.setCursor(14, 0);


  if (sekunden < 10)
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:0"); lcd.print(sekunden);
    }
  }
  else
  {
    if (minuten == 0)
    { lcd.setCursor(14, 0);
      lcd.print("00:"); lcd.print(sekunden);
    }
  }

  minuten = ((stunden * 60) + minutenwert);

  if ((kochzeit - minuten == hopfengabe[h]) && (h < hanz))
  {
    lcd.setCursor(0, 2); lcd.print(h + 1); lcd.print(". H-Gabe");
    h++;
    signalton(2000);
  }

  if ((sekunden + 10) % 30 == 0) {
    lcd.setCursor(0, 2);
    lcd.print("          ");
  }


  if ((minuten < 10) && (minuten > 0))
  {
    lcd.setCursor(14, 0);
    lcd.print("0"); lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if ((minuten >= 10) && (minuten < 100))
  {
    lcd.setCursor(14, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  if (minuten >= 100)
  {
    lcd.setCursor(13, 0);
    lcd.print(minuten); lcd.print(":");
    if (sekunden < 10) {
      lcd.print("0");
      lcd.print(sekunden);
    } else lcd.print(sekunden);
  }

  kochzeit = drehen; //5min-Sprünge

  if (kochzeit <= 20)
  {
    kochzeit = 20;
    drehen = 20;
  }
  if (kochzeit >= 180)
  {
    kochzeit = 180;
    drehen = 180;
  }

  if (minuten >= kochzeit)     //Kochzeitende
  {
    modus = 80;                //Abbruch nach Rufalarm
    rufmodus = modus;
    modus = 31;
    regelung = 0;              //Regelung aus
    heizung = 0;               //Heizung aus
    y = 0;
    braumeister[y] = 2;
    anfang = 0;
  }

}
//------------------------------------------------------------------


void khwert() //Modus=11
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Faktor Heizen");
    lcd.setCursor(0, 1);
    lcd.print("kh = ");

    kh =  EEPROMReadlong(90);
    anfang = 1;
    drehen = kh;
    dclick = false;
  }

  if (drehen < 0) drehen = 200;
  if (drehen > 200) drehen = 0;


  kh  = drehen;
  lcd.setCursor(5, 1); lcd.print(kh / 10); lcd.print(' ');




  if (ButtonPressed)
    {  
      if (EEPROMReadlong(90) != drehen) EEPROMWritelong(90, drehen);
      speichern();
      modus = 12;
      kh /= 10;
      anfang = 0;
    }

}

void ESVHeiz() //Modus=12
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Einschaltv. Heizen");
    lcd.setCursor(0, 1);
    lcd.print("ESVH = ");

    esvheizen =  EEPROMReadlong(86);
    anfang = 1;
    drehen = esvheizen / 1000;
  }

  if (drehen < 0) drehen = 60;
  if (drehen > 60) drehen = 0;

  lcd.setCursor(7, 1); lcd.print(drehen); lcd.print(" s ");

  esvheizen = drehen*1000L;

  if (ButtonPressed)
 
    {
      if (EEPROMReadlong(86) != esvheizen) EEPROMWritelong(86, esvheizen);
      speichern();
      modus = 13;
      anfang = 0;
    }

}

void kkwert() //Modus=13
{

  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Faktor Kuehlen");
    lcd.setCursor(0, 1);
    lcd.print("kk = ");

    kk =  EEPROMReadlong(94);
    anfang = 1;
    drehen = kk;
  }

  if (drehen < 0) drehen = 200;
  if (drehen > 200) drehen = 0;


  kk  = drehen;
  lcd.setCursor(5, 1); lcd.print(kk / 10); lcd.print(' ');




  if (ButtonPressed)
    {  
      if (EEPROMReadlong(94) != drehen) EEPROMWritelong(94, drehen);
      speichern();
      modus = 14;
      kk /= 10;
      anfang = 0;
    }

}


// Funktion Einschaltverzögerung Kühlen (Kompressorschutz)---------

void wartenKuehlen()  //Modus 14
{
  if (anfang == 0)
  {
    lcd.clear();
    lcd.setCursor(0, 0); lcd.print("Wartezeit Kuehlen");
    lcd.setCursor(0, 1);
    lcd.print("ESVK = ");
    kzeitverz = EEPROMReadlong(82);
    kzeitverz = kzeitverz / (60000);
    drehen = kzeitverz;
    anfang = 1;
  }
  else
  {

    kzeitverz = drehen;


    lcd.setCursor(7, 1); lcd.print(kzeitverz); lcd.print(" min  ");

      if (ButtonPressed)
      {
        EEPROMWritelong(82, kzeitverz * 60000);
        kzeitverz = kzeitverz * 60000;
        speichern();
        modus = 5;
        anfang = 0;
      }
  }
}


//-------------------------------------------------------------------------------------

void LogviewEinAus() //Modus=43
{

  if ((anfang == 0) || (dclick))
  {
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Logview Ein?");
    drehen = EEPROM.read(30);
    anfang = 1;
    dclick = false;
  }

  if (drehen > 1) drehen = 0;
  if (drehen < 0) drehen = 1;

  lcd.setCursor(0, 1); if (drehen == 1) lcd.print("J"); else lcd.print("N");

  headersend = drehen;

  
    if (ButtonPressed)
    {
      modus = 5;
      EEPROM.write(30, drehen);
      speichern();
      anfang = 0;
    }
}

//-------------------------------------------------------------------------------------

//LogView Studio -- Download unter : http://www.logview.info/forum/index.php?resources/

void print_logview() //Daten für Logview-Studio bereitstellen
{
  if (Serial.available() > 0)
  {
    char c = Serial.read();
    switch (c)
    {
      case 83:                                   // "S" -> Start
        Serial.println("Start");
        // Send Header information
        Serial.print("$N$;Maische Temp Logging\r\n");

        Serial.print("$C$;Sollwert[C];IstTemp [C]\r\n");
        headersend = true;
        break;
      case 69:                                   // "E" -> End
        Serial.println("Log ende");
        headersend = false;
        break;
    }
  }
  if (headersend)
  {
    Serial.print("$");
    Serial.print(sollwert);
    Serial.print(";");
    Serial.print(isttemp);
    Serial.print("\r\n");
  }
}

void signalton(int dauer)
{
  digitalWrite(schalterB, B_EIN);   // Alarm einschalten
  delay(dauer);
  digitalWrite(schalterB, B_AUS);   // Alarm ausschalten
}


void EEPROMWritelong(int address, long value)
{
  byte four = (value & 0xFF);
  byte three = ((value >> 8) & 0xFF);
  byte two = ((value >> 16) & 0xFF);

  EEPROM.write(address, four);
  EEPROM.write(address + 1, three);
  EEPROM.write(address + 2, two);
}


long EEPROMReadlong(long address)
{
  long four = EEPROM.read(address);
  long three = EEPROM.read(address + 1);
  long two = EEPROM.read(address + 2);

  return ((four << 0) & 0xFF) + ((three << 8) & 0xFFFF) + ((two << 16) & 0xFFFFFF);
}

void speichern()
{
  lcd.setCursor(0, 2);
  lcd.print("speichern");
  delay(500);
}


// Funktion Abbruch-------------------------------------------------
void funktion_abbruch()       // Modus 80
{
  regelung = 0;
  heizung = 0;

  if (rwerk == 1) ruehrerEinAus(0);
  wartezeit = -60000;
  digitalWrite(schalterH1, H_AUS);  //Heizung aus
  digitalWrite(schalterH2, H_AUS);
  digitalWrite(schalterB, B_AUS);   // ausschalten
  digitalWrite(schalterR12, R_AUS);   // ausschalten
  digitalWrite(schalterR230, R_AUS);   // ausschalten

  x = 1;                              //Rasteingaben
  h = 0;                              //Hopfeneingaben
  y = 1;
  n = 0;
  nachgussruf = false;
  headersend = false;                 //Logview wieder gesperrt
  pause = 0;
  schwelle_erreicht = false;         // für Kochbeginn Meldung

  first = true;
  kzeitverz = 0;                    //Kompressorverzögerung

  zeitz = true;                     // Zeit läuft nach Doppelclick weiter wenn zeitz = false

  anfang = 0;                       //Daten zurücksetzen
  rufmodus = 0;                     //bleiben erhalten
  drehen = sollwert;                  //Zuweisung für Funktion Temperaturregelung
  
if (altmodus == 1)
{
  if (knot) knot = false; else knot = true;  
  if (!knot) {if (EEPROM.read(98) != 1) EEPROM.write(98, 1); if (EEPROM.read(99) != drehen) EEPROM.write(99,drehen);lcd.setCursor(0,3);lcd.print("NOT->EIN");;delay(1000);} //Beim Neustart in Modus 0 , bei Stromausfall in Modus (1) Kühlen
  if (knot){lcd.setCursor(0,3);lcd.print("NOT->AUS");delay(1000);  //Notstart zurücksetzen
             if (EEPROM.read(98) == 1) EEPROM.write(98, 0);}  //nach Notfall- wieder Normalmodus
}             
          
//  altmodus = 0;                     //Merker alter Modus
  modus = 0;                        //Hauptmenue
  lcd.clear();
}
//------------------------------------------------------------------



